/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file ScrollingRect.hpp
/// @desc Scrolling Rectangle Widget
/// @edit 2024-10-11

#pragma once

#include <Event.h>
#include <J3ML/LinearAlgebra/Vector2.hpp>

namespace JUI
{
    /// A mixin helper class that enables a widget to be dragged around by the mouse.
    class Draggable {
    public:
        Event<> OnDragBegan;
        Event<> OnDragEnded;
        bool Dragging() const { return dragging;}
        virtual void SetDrag(bool b) {
            this->dragging = b;
        }
        virtual void StartDragging(const Vector2& point)
        {
            initial_drag_offset = point;
        }
        virtual void StopDragging() {}
        virtual void Update(float delta) {}
    protected:
        bool dragging = false;
        Vector2 initial_drag_offset = {0, 0};
    };
}


