/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2025 Redacted Software
/// This work is dedicated to the public domain.

/// @file Collapsible.cpp
/// @desc A rectangular widget with a clickable header that enables hiding and showing the contents.
/// @edit 2025-02-14
#pragma once

#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/TextButton.hpp>

namespace JUI {
    class Collapsible : public Rect {
    public:
        Collapsible();
        explicit Collapsible(Widget* parent);
        TextButton* Header();
        Rect* ContentBox();
        void Collapse();
        void Expand();
        [[nodiscard]] bool Collapsed() const;

        UDim HeaderHeight() const;
        void HeaderHeight(const UDim& value);

        void Title(const std::string& value);
        std::string Title() const;


    protected:
        TextButton* header = nullptr;
        Rect* content_box = nullptr;
        TextRect* lil_arrow = nullptr;
        bool collapsed = false;
        UDim2 saved_size;
        UDim header_height = Style::Collapsible::HeaderHeight;
        Tween* resize_anim = nullptr;
    };


}