#pragma once

#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/TextRect.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/Widgets/Text.hpp>

#include "../Style/TextStyler.hpp"
#include <JUI/Widgets/Slider.hpp>

#include "Window.hpp"

/// TODO: Move to a utility library or something...
/// Converts a set of R,G,B values to it's hexadecimal string representation.
std::string rgb2hex(int r, int g, int b, bool with_head);

namespace JUI {

    /// A JUI Widget that displays a HSV color input dialog.
    class ColorPicker : public Rect, public TextStyler {
    public:
        ColorPicker();

        explicit ColorPicker(Widget* parent);

        /// Invoked when the color value is changed by the user.
        /// @param Color4 The new color value.
        Event<Color4> OnColorValueChanged;

        /// Sets the font of the text elements contained in this widget.
        void Font(const JGL::Font &value) override;

        /// Sets the point size of the text elements contained in this widget.
        void TextSize(int size) override;

        /// Sets the point size of the text elements contained in this widget.
        void TextColor(const Color4 &color) override;




        void SetColorValue(const Color4& color);
        Color4 GetColorValue() const;


        float Hue() const { return hue;}
        float Sat() const { return sat;}
        float Val() const { return bri;}


    protected:
        /// Calculates the colors of the label, and the inverse color for the text.
        void RecomputeVisuals();

        /// Applies this widget's specific styling to it's slider sub-components.
        void StyleSlider(Slider* slider);

        JUI::Slider* hue_slider = nullptr;
        JUI::Slider* sat_slider = nullptr;
        JUI::Slider* bri_slider = nullptr;
        JUI::Text* hue_label = nullptr;
        JUI::Text* sat_label = nullptr;
        JUI::Text* bri_label = nullptr;
        JUI::TextRect* hex_label = nullptr;

        float hue = 0;
        float sat = 1.f;
        float bri = 1.f; // AKA val
    private:
    };

    class ColorPickerDialog : public Window {
        public:
        protected:
        private:

    };

}
