/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Scene.hpp
/// @desc Scene class - The root class of a menu that contains child elements.
/// @edit 2024-10-11

#pragma once
#include <JUI/Base/Widget.hpp>

namespace JUI
{
    ///
    class Scene : public Widget {
    public:
        Scene();
        ~Scene() override {}
        void Draw() override;

        Event<Vector2> MouseMoved;

        void GlobalUIScale(const Vector2& value);
        [[nodiscard]] Vector2 GlobalUIScale() const;

        void Update(float delta) override;

        [[nodiscard]] Vector2 GetAbsolutePosition() const override;

        [[nodiscard]] Vector2 GetAbsoluteSize() const override;

        bool ObserveMouseMovement(const Vector2 &latest_known_pos) override;

    protected:
        Vector2 ui_scale = Vector2(1, 1);
    };
}
