#include <JUI/Base/ImageBase.hpp>
#include <JGL/JGL.h>
#include <JUI/JUI.hpp>

using namespace JGL;

namespace JUI
{
    void ImageBase::Draw(const Vector2 &pos) {
        if (texture == nullptr) {
            UILogs("Attempt to draw ImageBase that has nullptr texture!");
            return;
        }

        // TODO: Support image rotation in the widget.
        J2D::DrawSprite(*texture, pos, 0, origin, scale, image_color);

    }

    void ImageBase::Draw(const Vector2 &pos, const Vector2 &size) {
        if (texture == nullptr) {
            UILogs("Attempt to draw ImageBase that has nullptr texture!");
            return;
        }

        Vector2 overridden_scale = size / Vector2(texture->GetDimensions());


        // TODO: Support image rotation in the widget.
        J2D::DrawSprite(*texture, pos, 0, origin, overridden_scale, image_color);
    }

    ImageBase::ImageBase()
    {
        this->texture = nullptr;
    }

    ImageBase::ImageBase(JGL::RenderTarget* content) : ImageBase() {
        this->Content(content);
    }

    ImageBase::ImageBase(JGL::Texture* tex) : ImageBase()
    {
        this->Content(tex);
    }

    void ImageBase::Content(const JGL::Texture *content) {
        this->texture = new JGL::RenderTarget(content);
    }

    void ImageBase::Content(JGL::RenderTarget *content) {
        this->texture = content;
    }


    void ImageBase::Color(const Color4 &newColor) { this->image_color = newColor;}

    void ImageBase::Scale(const Vector2 &newScale) { scale = newScale;}

    void ImageBase::Origin(const Vector2 &newOrigin) { origin = newOrigin; }

    const JGL::Texture *ImageBase::Content() const { return texture->GetTexture();}

    Color4 ImageBase::Color() const { return image_color;}

    Vector2 ImageBase::Scale() const { return scale;}

    Vector2 ImageBase::Origin() const { return origin;}


}

