#include <JUI/Base/RectBase.hpp>
#include <JGL/JGL.h>

using namespace JGL;

namespace JUI {
    RectBase::RectBase() = default;

    bool RectBase::GetClipsDescendants() const { return clips_descendants;}

    void RectBase::SetClipsDescendants(bool clipping) { clips_descendants = clipping; }

    void RectBase::BGColor(const Color4 &col) { bg_color = col; }

    void RectBase::BorderColor(const Color4 &col) { border_color = col; }

    void RectBase::BorderWidth(float w) { border_width = w; }

    float RectBase::GetBorderWidth() const { return border_width; }

    Color4 RectBase::GetBorderColor() const { return border_color; }

    Color4 RectBase::BGColor() const { return bg_color; }

    void RectBase::SetBorderStyling(const Color4 &color, float width) {
        BorderColor(color);
        BorderWidth(width);
    }


    void RectBase::Draw(const Vector2& abs_pos, const Vector2& abs_size) {
        Draw(bg_color, border_color, abs_pos, abs_size);
    }

    void RectBase::Draw(const Color4& bgColor, const Color4& borderColor, const Vector2 &abs_pos, const Vector2 &abs_size) {

        RectBase::Draw(bgColor, borderColor, abs_pos, abs_size, corner_rounding_radius, border_width, border_mode, corner_mode);

    }

    void RectBase::DrawOutline(const Color4 &color, const Vector2 &pos, const Vector2 &size, float rounding,
        float borderWidth, enum JUI::BorderMode borderMode, enum JUI::CornerRoundingMode cornerRoundingMode) {
        // Outline rect - compute the size change to fit the border accurately.
        Vector2 border_offset = {0, 0};

        /// Border is too small, don't draw it.
        if (borderWidth <= 0) return;

        if (borderMode == BorderMode::Inset)
            border_offset = {-borderWidth/2.f, -borderWidth/2.f};
        if (borderMode == BorderMode::Middle)
            border_offset =  {0, 0};
        if (borderMode == BorderMode::Outline)
            border_offset = {borderWidth/2.f, borderWidth/2.f};

        if (cornerRoundingMode == CornerRoundingMode::None || rounding <= 0)
            J2D::OutlineRect(color, pos - border_offset, size + (border_offset * 2), borderWidth);
        else if (cornerRoundingMode == CornerRoundingMode::Rounded)
            J2D::OutlineRoundedRect(color, pos - border_offset, size + (border_offset*2), rounding, borderWidth);
        else if (cornerRoundingMode == CornerRoundingMode::Chamfer)
            J2D::OutlineChamferRect(color, pos - border_offset, size + (border_offset*2), rounding, borderWidth);

    }

    void RectBase::DrawBG(const Color4 &color, const Vector2 &pos, const Vector2 &size, float rounding,
                          enum JUI::CornerRoundingMode rounding_mode) {


        if (rounding_mode == CornerRoundingMode::None || rounding <= 0)
            J2D::FillRect(color, pos, size);
        else if (rounding_mode == CornerRoundingMode::Rounded)
            J2D::FillRoundedRect(color, pos, size, rounding);
        else if (rounding_mode == CornerRoundingMode::Chamfer)
            J2D::FillChamferRect(color, pos, size, rounding);

    }

    void RectBase::Draw(const Color4 &bgColor, const Color4 &fgColor, const Vector2 &pos, const Vector2 &size,
        float rounding, float borderWidth, enum JUI::BorderMode borderMode,
        enum JUI::CornerRoundingMode rounding_mode) {
        DrawBG(bgColor, pos, size, rounding, rounding_mode);
        DrawOutline(fgColor, pos, size, rounding, borderWidth, borderMode, rounding_mode);
    }

    void RectBase::BorderMode(const enum BorderMode &mode) {
        border_mode = mode;
    }

    BorderMode RectBase::BorderMode() const { return border_mode; }

    void RectBase::CornerRounding(float radius) {
        corner_rounding_radius = radius;
    }


    float RectBase::CornerRounding() const { return corner_rounding_radius; }


}