#include <JUI/Base/TextBase.hpp>
#include <JGL/JGL.h>
using namespace JUI;

void TextBase::Content(const std::string& content) { this->content = content; state_redraw = true; }

void TextBase::OutlineColor(const Color4& color) { this->outline_color = color; }

void TextBase::HorizontalTextAlign(const TextAlign::H& align) { this->h_align = align;}

void TextBase::VerticalTextAlign(const TextAlign::V& align) { this->v_align = align; }

void TextBase::Align(const TextAlign::H& h_align, const TextAlign::V& v_align) {
    HorizontalTextAlign(h_align);
    VerticalTextAlign(v_align);
}

void TextBase::AlignLeft() { HorizontalTextAlign(TextAlign::H::Left); }

void TextBase::AlignRight() { HorizontalTextAlign(TextAlign::H::Right); }

void TextBase::AlignTop() { VerticalTextAlign(TextAlign::V::Top); }

void TextBase::AlignBottom() { VerticalTextAlign(TextAlign::V::Bottom); }

void TextBase::AlignCenterBoth() {
    VerticalTextAlign(TextAlign::V::Center);
    HorizontalTextAlign(TextAlign::H::Center);
}

void TextBase::Center() { AlignCenterBoth(); }

void TextBase::AlignCenterHorizontally() {
    HorizontalTextAlign(TextAlign::H::Center);
}

void TextBase::AlignCenterVertically() {
    VerticalTextAlign(TextAlign::V::Center);
}

void TextBase::WordWrap(bool wrap) { word_wrap = wrap; }

void TextBase::Draw(const Vector2& abs_pos, const Vector2& abs_size, const std::string& content, unsigned int size, const Color4& color) {
    // Calculate how much to origin the text based on alignment.
    float align_x = abs_pos.x;
    float align_y = abs_pos.y;

    std::string start = content;
    std::vector<std::string> lines;

    /*for (int i = 0; i < start.length(); i++) {
        std::string substr = start.substr(0, i);
        auto sub_bounds = this->GetFont().MeasureString(substr, size);
        if (sub_bounds.x > abs_size.x)
    }*/

    auto bounds = TextStyler::Font().MeasureString(content, size);
    // Do nothing if there is no text.
    if (bounds.x == 0 || bounds.y == 0)
        return;

    // Render from left-side of boundary.
    if (h_align == TextAlign::H::Left)
        align_x = abs_pos.x;
    // Render from horizontal center, origin by half of the text's width.
    if (h_align == TextAlign::H::Center)
        align_x = abs_pos.x + (abs_size.x / 2) - (bounds.x / 2);
    // Render from right-side of boundary, origin by text width.
    if (h_align == TextAlign::H::Right)
        align_x = abs_pos.x + abs_size.x - bounds.x;
    // Render from top of boundary.
    if (v_align == TextAlign::V::Top)
        align_y = abs_pos.y;
    // Render from vertical center, origin by half of the text's height.
    if (v_align == TextAlign::V::Center)
        align_y = abs_pos.y + (abs_size.y / 2) - (bounds.y / 2);
    // Render from bottom of boundary, origin by the text's height.
    if (v_align == TextAlign::V::Bottom)
        align_y = abs_pos.y + abs_size.y - bounds.y;

    float scale = 1.f;
    bool use_render_target = false;

    // Has to be checked because this function lets you pass in parameters instead
    // of using the ones we have saved. Which again, defeats the point. - Redacted.
    if (content == this->content) {
        // if Render Target needs updating.
        if (state_redraw) {
            text_canvas->Resize({(int) bounds.x, (int) bounds.y});
            J2D::Begin(text_canvas, nullptr, true);
                //J2D::FillRect(Colors::Red, {0,0}, bounds);
                J2D::DrawString(color, content, 0, 0, size, scale, this->font);
            J2D::End();
            state_redraw = false;
        }
        use_render_target = true;
    }

    Vector2 text_pos = {align_x, align_y};
    //J2D::Begin();
    //J2D::OutlineRect(color, text_pos, bounds); // Draw bounding box for debugging.
    use_render_target ? J2D::DrawRenderTarget(text_canvas, {align_x, align_y})
    : J2D::DrawString(color, content, align_x, align_y, size, scale, this->font);
    //J2D::End();
}

void TextBase::Draw(const Vector2& abs_pos, const Vector2& abs_size) {
    Draw(abs_pos, abs_size, this->content, this->text_size, this->text_color);

}

std::string TextBase::Content() const { return content; }


Color4 TextBase::OutlineColor() const { return outline_color; }

TextAlign::H TextBase::HorizontalTextAlign() const { return h_align; }

TextAlign::V TextBase::VerticalTextAlign() const { return v_align; }

Vector2 TextBase::TextBounds() { return TextStyler::Font().MeasureString(content, text_size); }


void TextBase::Font(const JGL::Font& font) {
    TextStyler::Font(font);
    state_redraw = true;
}

void TextBase::TextSize(int size) {
    TextStyler::TextSize(size);
    state_redraw = true;
}

void TextBase::TextColor(const Color4& color) {
    TextStyler::TextColor(color);
    state_redraw = true;
}

void TextBase::Update(float delta) {}