#include <JUI/Widgets/Collapsible.hpp>

namespace JUI {
    Collapsible::Collapsible() : Rect() {
        Name("Collapsible");

        header = new TextButton(this);
        header->Size({100_percent, header_height});
        header->Content("Collapsible");
        header->BaseBGColor(Colors::DarkGray);
        header->BGColor(Colors::DarkGray);
        header->Center();
        //header->
        header->OnClickEvent += [this] (auto a, auto b) {
            if (Collapsed())
                Expand();
            else
                Collapse();
        };

        lil_arrow = new TextRect(header);
        lil_arrow->Size({header_height, header_height});
        lil_arrow->BGColor(Colors::Transparent);
        lil_arrow->Content("/\\");
        lil_arrow->BorderWidth(0);

        content_box = new Rect(this);
        content_box->BGColor(Colors::Transparent);
        content_box->BorderColor(Colors::Transparent);
        content_box->BorderWidth(0);
        content_box->Position({0_px, header_height});
        content_box->Size({100_percent, 100_percent - header_height});

    }

    void Collapsible::Collapse() {
        //Size(saved_size);
        lil_arrow->Content("\\/");
        content_box->Visible(false);
        if (resize_anim != nullptr && !resize_anim->HasCompleted()) {
            resize_anim->Cancel();
        } else {
            saved_size = Size();
        }

        resize_anim = TweenSize(UDim2(saved_size.X, header_height), {
            .time = 0.25f
        });
        collapsed = true;
    }

    void Collapsible::Expand() {
        lil_arrow->Content("/\\");
        content_box->Visible(true);
        if (resize_anim != nullptr  && !resize_anim->HasCompleted())
            resize_anim->Cancel();


        resize_anim = TweenSize(saved_size, {
            .time = 0.25f
        });
        collapsed = false;
    }

    bool Collapsible::Collapsed() const { return collapsed; }

    Collapsible::Collapsible(Widget *parent) : Collapsible() {
        this->Parent(parent);
    }

    TextButton *Collapsible::Header() { return header; }

    Rect *Collapsible::ContentBox() {return content_box; }

    UDim Collapsible::HeaderHeight() const { return header_height;}

    void Collapsible::HeaderHeight(const UDim &value) { header_height = value; }

    void Collapsible::Title(const std::string &value) {
        header->Content(value);
    }

    std::string Collapsible::Title() const { return header->Content(); }
}