#include <JUI/Widgets/ContextMenu.hpp>

namespace JUI {
    ContextMenu::ContextMenu(): Rect() {
        Name("ContextMenu");
        this->BGColor(Colors::White);
        //this->Margin(2_px);
        this->Size({200, 200, 0, 0});
        layout = new VerticalListLayout(this);

        MouseExit += [this] (auto _) {
            if (!Pinned())
                Visible(false);
        };
    }

    ContextMenu::ContextMenu(Widget *parent): ContextMenu() {
        this->Parent(parent);
    }

    void ContextMenu::TextSize(int value) {
        TextStyler::TextSize(value);

        for (auto* btn : buttons)
            btn->TextSize(value);
        for (auto* submenu: submenus)
            submenu->TextSize(value);
    }

    void ContextMenu::TextColor(const Color4 &color) {
        TextStyler::TextColor(color);

        for (auto* btn : buttons)
            btn->TextColor(color);
        for (auto* submenu: submenus)
            submenu->TextColor(color);

    }

    void ContextMenu::Font(const JGL::Font &value) {
        TextStyler::Font(value);
        for (auto* btn : buttons)
            btn->Font(value);
        for (auto* submenu: submenus)
            submenu->Font(value);
    }

    float GuesstimateTextWidth(const std::string& text, int ptSize) {
        int length = text.length();
        return length * (ptSize/2.f);
    }

    TextButton * ContextMenu::AddButton(const std::string &name) {
        auto* line_item = new TextButton(layout);
        line_item->Style(font, 16, Colors::Black);
        line_item->Font(font);
        line_item->Content(name);
        line_item->TextSize(16);
        line_item->TextColor(Colors::Black);
        line_item->Size({0, 20, 1, 0});

        float line_width = GuesstimateTextWidth(line_item->Content(), line_item->TextStyler::TextSize());

        if (line_width > estimated_width)
            estimated_width = line_width + 10;

        // TODO: Temp Hack: Implement Auto-size.
        aggregate_height += 20;
        this->size.X.Pixels = estimated_width;
        this->size.Y.Pixels = aggregate_height;


        buttons.push_back(line_item);

        return line_item;
    }

    TextButton * ContextMenu::AddButton(const std::string &name, const std::function<void()> &callback) {

        auto* btn = AddButton(name);

        // TODO: Is this memory safe at all?
        btn->OnReleaseEvent += [this, btn, callback]  (auto a, auto b, bool still_focus) mutable {
            if (still_focus && this->IsVisible() && btn->IsMouseInside())
                callback();
        };
        return btn;
    }

    Separator * ContextMenu::AddSeparator(const UDim& size) {
        auto* separator = new Separator(layout);
        separator->Size({100_percent, size});

        // TODO: Temp Hack: Implement Auto-size.
        aggregate_height += separator->GetAbsoluteSize().y;
        this->size.X.Pixels = estimated_width;
        this->size.Y.Pixels = aggregate_height;

        return separator;
    }

    ContextMenu * ContextMenu::AddSubmenu(const std::string &name) {
        auto* btn = AddButton(name);

        // TODO: For auto-hiding, check if the widget is over a submenu as well.
        // TODO: Duplicated code with UtilityBar.cpp
        auto* submenu = new JUI::ContextMenu(btn);
        submenu->AnchorPoint({0, 0});
        submenu->Position({100_percent, 0_percent});
        submenu->Visible(false);

        btn->OnReleaseEvent += [this, submenu] (auto a, auto b, bool still_there) mutable {
            if (still_there && this->IsVisible()) {
                this->Pin();
                submenu->Visible(true);
            }

        };

        submenu->MouseExit += [this, submenu] (auto _) {
                if (!submenu->Pinned())
                    this->Unpin();
        };


        submenus.push_back(submenu);

        return submenu;
    }

    void ContextMenu::CloseOnHoverLoss(bool value) { close_on_hover_loss = value;}

    bool ContextMenu::CloseOnHoverLoss() const { return close_on_hover_loss; }

    void ContextMenu::Update(float delta) {
        Rect::Update(delta);

        if (parent->IsMouseInside() || IsMouseInside() || IsMouseInsideDescendants())
            mouse_outside_tests = 0;
        else
            mouse_outside_tests++;

        if (mouse_outside_tests >= 3)
            PresumeMouseFocusLost();
    }
}
