#include <JUI/Widgets/FpsGraph.hpp>

namespace JUI {
    void FpsGraph::RenderDataPoints() {
        J2D::Begin(canvas, nullptr, true);

        DataPoint prev = data[0];
        for (auto& data_pt : data) {

            //J2D::DrawGradientLine(prev.color, data_pt.color, prev.pos, data_pt.pos, 1);
            J2D::DrawLine(data_pt.color, data_pt.pos, {data_pt.pos.x, graph_height }, 1);

            prev = data_pt;
        }

        float target_delta          = 1.f / target_fps;
        float target_line_height    = target_delta * y_axis_scale;
        float target_fps_rounded    = Math::Floor(target_fps);
        float target_delta_rounded  = Math::Round(target_delta, 1);

        std::string target_fps_label_text = std::format("{} FPS / {}ms (Target)", target_fps_rounded, target_delta_rounded);

        J2D::DrawString(Colors::Black, target_fps_label_text, 0, graph_height - target_line_height, 10, 1.f);
        J2D::DrawLine(Colors::White, {0, graph_height - target_line_height}, {GetAbsoluteSize().x, graph_height - target_line_height}, 1);
        J2D::End();
    }

    FpsGraph::FpsGraph(): JUI::ImageRect() {

        BGColor(Colors::Transparent);
        BorderColor(Colors::Transparent);

        for (int i = 0; i < sample_history; i++) {
            Plot({i / 1.f, 0.f}, Colors::Black);
        }

        canvas = new JGL::RenderTarget(Vector2i(graph_width, graph_height));

        RenderDataPoints();

        Content(canvas);
    }

    FpsGraph::FpsGraph(Widget *parent): FpsGraph() {
        Parent(parent);
    }

    void FpsGraph::Plot(const Vector2 &pt, const Color4 &col) {
        data.push_back(DataPoint{ pt, col });
    }

    void FpsGraph::Plot(const DataPoint &data_pt) {
        data.push_back(data_pt);
    }

    void FpsGraph::Update(float delta) {
        ImageRect::Update(delta);
        data.erase(data.begin());
        for (auto& data_pt : data) {
            data_pt.pos.x -= 1;
        }

        Color4 color = Colors::Green;
        if (delta > 1.f/HighFPSRange())
            color = Colors::Blue;
        if (delta > 1.f/TargetFPSRange())
            color = Colors::Yellow;
        if (delta > 1.f/LowFPSRange())
            color = Colors::Red;
        if (delta > 1.f/UnplayableFPSRange())
            color = Colors::Purples::Magenta;
        if (delta > 1.f/5.f)
            color = Colors::Black;


        Plot({(data.size()-1.f),  graph_height - (delta*y_axis_scale)}, color);

        RenderDataPoints();
        Content(canvas);
    }

    void FpsGraph::SetupAsPseudoDockedElementAtBottomOfScreen() {
        this->Size({100_percent, 50_px});
        this->AnchorPoint({1, 1});
        this->Position({100_percent, 100_percent});
        this->BGColor(Colors::Transparent);
        this->BorderColor(Colors::Transparent);
    }

    void FpsGraph::ShowTargetFPS(bool show) { show_target_fps_bar = show;}

    bool FpsGraph::ShowTargetFPS() const { return show_target_fps_bar;}

    void FpsGraph::TargetFPS(float fps) {target_fps = fps;}

    float FpsGraph::TargetFPS() const { return target_fps; }

    float FpsGraph::HighFPSRange() const { return target_fps * high_fps_range_factor;}

    float FpsGraph::TargetFPSRange() const { return target_fps * target_fps_range_factor;}

    float FpsGraph::SubparFPSRange() const {return target_fps * subpar_fps_range_factor;}

    float FpsGraph::LowFPSRange() const { return target_fps * low_fps_range_factor;}

    float FpsGraph::UnplayableFPSRange() const { return target_fps * unplayable_fps_range_factor;}

    float FpsGraph::SlideshowFPSRange() const { return target_fps * slideshow_fps_range_factor;}

    FpsGraphWindow::FpsGraphWindow(): Window() {
        Name("FPSGraphWindow");
        Title("FPS Graph");

        Size({500_px, 70_px});
        this->SetResizable(false);


        data_graph = new FpsGraph(this->Content());
        data_graph->Size({500_px, 50_px});

    }

    FpsGraphWindow::FpsGraphWindow(Widget *parent): FpsGraphWindow() {
        this->Parent(parent);
    }
}
