#include <JUI/Widgets/ListLayout.hpp>

namespace JUI
{
    ListLayout::ListLayout() : LayoutContainer() {
        Name("ListLayout");
    }
    ListLayout::ListLayout(Widget* parent) : ListLayout() {
        Parent(parent);
    }

    VerticalListLayout::VerticalListLayout() : ListLayout() {}
    VerticalListLayout::VerticalListLayout(Widget* parent) : VerticalListLayout() {
        this->Parent(parent);
    }

    struct {
        bool operator()(Widget* a, Widget* b) const { return a->LayoutOrder() > b->LayoutOrder();}
    } layoutOrderSort;

    void VerticalListLayout::ApplyLayout() {

        content_height = 0;

        std::sort(children.begin(), children.end(), layoutOrderSort);

        if (layout == LayoutOrder::V::TOP)
        {
            int consumed_height = 0;
            for (auto& child_widget : children)
            {
                // TODO: Implement widget.LayoutOrder property
                // TODO: Sort children by LayoutOrder
                consumed_height += pad_top.Pixels;
                content_height += pad_top.Pixels;

                child_widget->Position({0, consumed_height, 0, 0});

                consumed_height += child_widget->GetAbsoluteSize().y;
                content_height += child_widget->GetAbsoluteSize().y;

                consumed_height += pad_bottom.Pixels;
                content_height += pad_bottom.Pixels;
            }


        }

        if (layout == LayoutOrder::V::BOTTOM)
        {
            int consumed_height = this->GetAbsoluteSize().y;
            for (auto& child_widget : children)
            {
                // TODO: Implement widget.LayoutOrder property
                // TODO: Sort children by LayoutOrder

                consumed_height -= pad_top.Pixels;
                content_height += pad_top.Pixels;

                consumed_height -= child_widget->GetAbsoluteSize().y;
                content_height += child_widget->GetAbsoluteSize().y;

                child_widget->Position({0, consumed_height, 0, 0});


                consumed_height -= pad_bottom.Pixels;
                content_height += pad_bottom.Pixels;
            }
        }
    }

    void VerticalListLayout::LayoutOrder(LayoutOrder::V order) {
        layout = order;
    }

    LayoutOrder::V VerticalListLayout::LayoutOrder() const { return layout;}

    float VerticalListLayout::CurrentContentHeight() const {
        return content_height;
    }

    HorizontalListLayout::HorizontalListLayout() : ListLayout() {}


    HorizontalListLayout::HorizontalListLayout(Widget* parent) : HorizontalListLayout() {
        Parent(parent);
    }

    void HorizontalListLayout::ApplyLayout() {
        // TODO: Implement widget.LayoutOrder and sort by that number.

        std::sort(children.begin(), children.end(), layoutOrderSort);

        if (layout == LayoutOrder::H::LEFT)
        {

            int consumed_width = 0;
            for (auto &child_widget : children)
            {

                consumed_width += pad_left.Pixels;
                content_width += pad_left.Pixels;

                child_widget->Position({consumed_width, 0, 0, 0});

                consumed_width += child_widget->GetAbsoluteSize().x;
                content_width += child_widget->GetAbsoluteSize().x;

                consumed_width += pad_right.Pixels;
                content_width += pad_right.Pixels;
            }
        }

        if (layout == LayoutOrder::H::RIGHT)
        {
            int consumed_width = this->GetAbsoluteSize().x;
            for (auto &child_widget : children)
            {
                consumed_width -= pad_left.Pixels;
                content_width += pad_left.Pixels;

                consumed_width -= child_widget->GetAbsoluteSize().x;
                content_width += child_widget->GetAbsoluteSize().x;

                child_widget->Position({consumed_width, 0, 0, 0});

                consumed_width -= pad_right.Pixels;
                content_width += pad_right.Pixels;
            }
        }
    }

    void HorizontalListLayout::LayoutOrder(LayoutOrder::H order) {
        layout = order;
    }

    LayoutOrder::H HorizontalListLayout::LayoutOrder() const {
        return layout;
    }

    float HorizontalListLayout::CurrentContentWidth() const { return content_width;}
}