#include <JGL/JGL.h>
#include <JUI/Widgets/Rect.hpp>
#include <jlog/Logger.hpp>

namespace JUI {
    Rect::Rect(): Widget(), RectBase() {
        Name("Rect");
    }

    Rect::Rect(Widget *parent): Rect() {
        this->Parent(parent);
    }


    void Rect::PreDraw()
    {
        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        auto root_size = GetFamilyTreeRoot()->GetAbsoluteSize();

        if (clips_descendants) {
            clip_was_enabled = glIsEnabled(GL_SCISSOR_TEST);
            if (clip_was_enabled)
                glGetIntegerv(GL_SCISSOR_BOX, old_scissor_bounds);

            float presumed_screen_height = viewport_size.y;
            glScissor(abs_pos.x, presumed_screen_height-abs_size.y-abs_pos.y, abs_size.x, abs_size.y);
            glEnable(GL_SCISSOR_TEST);
        }
    }

    void Rect::PostDraw()
    {
        // Returns clip to previous state
        if (clips_descendants)
        {
            glScissor(old_scissor_bounds[0], old_scissor_bounds[1], old_scissor_bounds[2], old_scissor_bounds[3]);

            if (!clip_was_enabled) {}
            glDisable(GL_SCISSOR_TEST);

        }
    }

    void Rect::InnerDraw() {
        RectBase::Draw(GetAbsolutePosition(), GetAbsoluteSize());
    }

    void Rect::Update(float delta) {
        if (IsMouseInside() && !mouse_inside_debounce) {
            MouseEnter.Invoke(last_known_mouse_pos);
            mouse_inside_debounce = true;
        }

        if (!IsMouseInside() && mouse_inside_debounce) {
            MouseExit.Invoke(last_known_mouse_pos);
            mouse_inside_debounce = false;
        }

        Widget::Update(delta);
    }

    AABB2D Rect::GetActualRenderBounds() const {

        Vector2 border = {border_width, border_width};

        if (border_mode == BorderMode::Inset)
            return { GetAbsolutePosition(), GetAbsoluteSize()};

        if (border_mode == BorderMode::Outline)
            return {GetAbsolutePosition() - border, GetAbsoluteSize() + border};

        if (border_mode == BorderMode::Middle)
            return { GetAbsolutePosition() - (border/2), GetAbsoluteSize() + (border/2)};

        return { GetAbsolutePosition(), GetAbsoluteSize()};
    }

    void Rect::Draw() {
        Widget::Draw();
    }
}


