#include <JUI/Widgets/Separator.hpp>


namespace JUI {
    Separator::Separator(): Widget() {
        Name("Separator");
    }

    void Separator::UpdateSize() {
        if (orientation == Orientation::Horizontal)
            Size({100_percent, spacing});

        if (orientation == Orientation::Vertical)
            Size({spacing, 100_percent});
    }

    Separator::Separator(Widget *parent, const enum JUI::Orientation &orientation, const UDim &spacing,
        const enum JUI::LineFillMode &line_mode, const Color4 &line_color, float line_thickness): Separator() {
        this->orientation = orientation;
        this->spacing = spacing;
        this->line_mode = line_mode;
        this->line_color = line_color;
        this->line_thickness = line_thickness;
        UpdateSize();
        Parent(parent);

    }


    void Separator::Orient(const enum JUI::Orientation &orientation) {
        this->orientation = orientation;

        UpdateSize();
    }

    void Separator::LineFillMode(const enum JUI::LineFillMode &line_mode) {
        this->line_mode = line_mode;
    }

    Color4 Separator::LineColor() const { return line_color; }

    void Separator::LineColor(const Color4 &color) {
        line_color = color;
    }

    float Separator::GetAbsoluteSpacing() const {
        if (parent == nullptr) {
            // TODO: This function should in theory not be called in this case.
            return 0.f;
        }
        auto abs_parent_size = parent->GetAbsoluteSize();
        if (orientation == ::JUI::Orientation::Horizontal) {
            return spacing.Pixels + (spacing.Scale * abs_parent_size.y);
        }

        if (orientation == Orientation::Vertical) {
            return spacing.Pixels + (spacing.Scale * abs_parent_size.x);
        }

        // This control path should never be reached.
        return 0.f;
    }

    void Separator::InnerDraw() {

        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        Vector2 start = abs_pos;
        Vector2 end = abs_size;

        // TODO: Factor in padding.

        float spacing = GetAbsoluteSpacing();

        if (orientation == Orientation::Horizontal) {
            start = abs_pos + Vector2(0, abs_size.y / 2.f);
            end = start + Vector2(abs_size.x, 0);
        }

        if (orientation == Orientation::Vertical) {
            start = abs_pos + Vector2(abs_size.x / 2.f, 0);
            end = start + Vector2(0, abs_size.y);
        }



        if (line_mode == LineFillMode::Dotted)
            J2D::DrawDottedLine(line_color, start, end, line_spacing, line_thickness);

        if (line_mode == LineFillMode::Dashed)
            J2D::DrawDashedLine(line_color, start, end, line_spacing, line_spacing, line_thickness);

        if (line_mode == LineFillMode::Solid)
            JGL::J2D::DrawLine(line_color, start, end, line_thickness);
    }

    enum Orientation Separator::Orientation() const { return orientation;}

    enum LineFillMode Separator::LineFillMode() const { return line_mode;}
}