#include <JUI/Widgets/UtilityBar.hpp>

namespace JUI {
    UtilityBar::UtilityBar(): Rect() {
        // TODO: Make a note that all JGL::Font members on widgets need to be initialized to JGL::Fonts::Jupiteroid inside the constructor.
        //font = JGL::Fonts::Jupiteroid;

        this->Size({0, 20, 1, 0});
        this->Position({0,0,0,0});
        this->BGColor(Colors::White);
        this->BorderColor(Colors::Blues::CornflowerBlue);
        this->BorderWidth(2);
        this->Margin(2_px);
        this->BorderMode(BorderMode::Outline);
        layout = new HorizontalListLayout(this);
        //layout->PaddingLeft(2_px);
        layout->PaddingRight(2_px);
        //layout->PaddingRight(2_px);
    }

    UtilityBar::UtilityBar(Widget *parent): UtilityBar() {
        this->Parent(parent);
    }

    ContextMenu * UtilityBar::AddSubmenu(const std::string &name) {
        auto* btn = AddButton(name);


        auto* submenu = new JUI::ContextMenu(btn);
        //submenu->AnchorPoint({0, 1});
        // TODO: Fix AnchorPoint behavior!!
        submenu->Position({0_percent, 100_percent});
        submenu->Visible(false);

        btn->OnClickEvent += [this, submenu] (auto a, auto b) mutable {
            if (this->IsVisible())
                submenu->Visible(true);
        };

        submenus.push_back(submenu);

        return submenu;
    }

    TextButton * UtilityBar::AddButton(const std::string &name) {
        // TODO: Refactor this to actually be able to use a set-font later.
        auto str_width = Fonts::Jupiteroid.MeasureString(name, 14);

        auto* btn = new TextButton(layout);
        btn->Font(Fonts::Jupiteroid);
        btn->TextSize(14);
        btn->TextColor(Colors::Black);
        btn->Size({static_cast<int>(str_width.x)+16, 0, 0, 1});   
        btn->BorderWidth(0.f);
        btn->Content(name);

        buttons.push_back(btn);

        return btn;
    }

    TextButton * UtilityBar::AddButton(const std::string &name, const std::function<void()> &callback) {
        auto* button = AddButton(name);

        button->OnClickEvent += [this, button, callback] (auto a, auto b) mutable {
            if (this->IsVisible() && button->IsMouseInside())
                callback();
        };
        return button;
    }

    Separator * UtilityBar::AddSeparator(const UDim& size) {
        auto* separator = new Separator(layout);

        separator->Size({100_percent, size});


        return new Separator(layout);
    }

    void UtilityBar::Font(const JGL::Font &value) {
        TextStyler::Font(value);
        for (auto* btn : buttons)
            btn->Font(value);

        for (auto* submenu : submenus)
            submenu->Font(value);
    }

    void UtilityBar::TextSize(int value) {
        TextStyler::TextSize(value);
        for (auto* btn : buttons)
            btn->TextSize(value);

        for (auto* submenu : submenus)
            submenu->TextSize(value);
    }

    void UtilityBar::TextColor(const Color4 &color) {
        TextStyler::TextColor(color);
        for (auto* btn : buttons)
            btn->TextColor(color);

        for (auto* submenu : submenus)
            submenu->TextColor(color);
    }
}
